#!/usr/bin/env python

"""
            A adesklet graph for multi-core CPUs
                        Version 0.0.1

Copyright (C) 2008 Angelo Arrifano <miknix@gmail.com>

------------------------------------------------------------------------

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""

import adesklets
import colorsys

class cpuStats:
	def __init__(self):
		self.used = []
		self.total = []
		fileStat = open('/proc/stat', 'r')
		try:
			for line in fileStat:
				cols = line.split()
				if cols[0][:3] != 'cpu':
					break
				used = int(cols[1]) + int(cols[2]) + int(cols[3])
				self.used.append(used)
				self.total.append(used + int(cols[4]))
		finally:
			fileStat.close()

	def get_usage(self):
		fileStat = open('/proc/stat', 'r')
		usage = []
		i = 0
		try:
			for line in fileStat:
				cols = line.split()
				if cols[0][:3] != 'cpu':
					break
				used = int(cols[1]) + int(cols[2]) + int(cols[3])
				total = used + int(cols[4])
				try:
					usage.append(float(used - self.used[i]) \
				            / float(total - self.total[i]) * 100 )
				except ZeroDivisionError:
					usage.append(0)
				self.used[i] = used
				self.total[i] = total
				i = i + 1
		finally:
			fileStat.close()
		return usage

class graph:
	def __init__(self, x, y, width, height, (r,g,b,a), dstbuffer):
		self.x = x
		self.y = y
		self.ww = width
		self.hh = height
		self.r = r
		self.g = g
		self.b = b
		self.a = a
		self.dstbuffer = dstbuffer
		self.buffer = adesklets.create_image(width, height)
		self.backbuffer = adesklets.create_image(width, height)
		adesklets.context_set_image(self.backbuffer)
		adesklets.blend_image_onto_image(dstbuffer, 0, \
		                                 self.x, self.y, self.ww, self.hh,
													0, 0, self.ww, self.hh)

	def __del__(self):
		adesklets.free_image(self.buffer)
		adesklets.free_image(self.backbuffer)
	
	def update(self, value):
		# Scroll graph, not sure if I can do this
		adesklets.context_set_image(self.buffer)
		adesklets.context_set_blend(False)
		adesklets.blend_image_onto_image(self.buffer, 1, \
		                                 1, 0, self.ww-1, self.hh,
													0, 0, self.ww-1, self.hh)
		
		# Draw bar
		adesklets.context_set_color(0, 0, 0, 0)
		adesklets.image_draw_line(self.ww-1, 0, self.ww-1, self.hh-1, 0)
		adesklets.context_set_blend(False)
		adesklets.context_set_color(self.r, self.g, self.b, self.a)
		adesklets.image_draw_line(self.ww-1, self.hh, self.ww-1, \
		               self.hh - (value * float(self.hh) / 100), 0)

		# Blend buffers
		adesklets.context_set_image(self.dstbuffer)
		adesklets.blend_image_onto_image(self.backbuffer, 0, \
		                                 0, 0, self.ww, self.hh,
													self.x, self.y, self.ww, self.hh)
		adesklets.context_set_blend(True)
		adesklets.blend_image_onto_image(self.buffer, 0, \
		                                 0, 0, self.ww, self.hh,
													self.x, self.y, self.ww, self.hh)

class CoreMon(adesklets.Events_handler):
	def __init__(self, config):
		self.stats = cpuStats()
		self.ww = config['width']
		self.hh = config['height']
		self.bg_color = config['bg_color']
		self.fg_color = config['fg_color']
		self.grid_x = config['grid_x']
		self.grid_y = config['grid_y']
		self.interval = config['interval']
		adesklets.Events_handler.__init__(self)

	def __del__(self):
		adesklets.free_image(self.buffer)
		for graph in self.graphs:
			graph.__del__()
		adesklets.Events_handler.__del__(self)

	def ready(self):
		adesklets.window_resize(self.ww, self.hh)
		adesklets.window_set_transparency(True)
		adesklets.window_show()

		# Create buffer for double-buffering
		self.buffer = adesklets.create_image(self.ww, self.hh)

		adesklets.context_set_image(self.buffer)
		adesklets.context_set_blend(False)

		# Clear background
		adesklets.context_set_color(self.bg_color[0], \
		                            self.bg_color[1], \
											 self.bg_color[2], \
											 self.bg_color[3])
		adesklets.image_fill_rectangle(0, 0, self.ww, self.hh)
		
		# Draw grid
		for i in range(1, self.grid_x):
			self._draw_line(((self.ww-4)/self.grid_x * i,  0, \
					          (self.ww-4)/self.grid_x * i, \
								 self.hh-1), self.bg_color, 40)
		for i in range(1, self.grid_y):
			self._draw_line((0, (self.hh-4)/self.grid_y * i + 3, \
					   self.ww-1, (self.hh-4)/self.grid_y * i + 3), \
						self.bg_color, 40)
		
		# Draw frame
		self._draw_line((0, 0, self.ww-1, 0), self.bg_color, 20)
		self._draw_line((1, 1, self.ww-2, 1), self.bg_color, 40)
		self._draw_line((self.ww-1, 0, self.ww-1, self.hh-1), self.bg_color, 80)
		self._draw_line((self.ww-2, 1, self.ww-2, self.hh-2), self.bg_color, 100)
		self._draw_line((self.ww-1, self.hh-1, 0, self.hh-1), self.bg_color, 80)
		self._draw_line((self.ww-2, self.hh-2, 1, self.hh-2), self.bg_color, 100)
		self._draw_line((0, self.hh-1, 0, 0), self.bg_color, 20)
		self._draw_line((1, self.hh-2, 1, 1), self.bg_color, 40)	

		# Create graphs
		usage = self.stats.get_usage()
		nr = len(usage) - 1
		self.graphs = []
		for i in range(nr):
			self.graphs.append(graph(2, (self.hh-4)/nr * i + 3, \
			                         self.ww-4, (self.hh-4)/nr - 1,
			                         self.fg_color[i % len(self.fg_color)], \
									       self.buffer))

	def alarm(self):
		usage = self.stats.get_usage()
		nr = len(usage) - 1
		for i in range(nr):
			self.graphs[i].update(usage[i+1])
		
		adesklets.context_set_image(0)
		adesklets.context_set_blend(False)
		adesklets.blend_image_onto_image(self.buffer, 1, \
		                                 0, 0, self.ww, self.hh,
													0, 0, self.ww, self.hh)
		return self.interval

	def _draw_line(self, (x1,y1,x2,y2), (r,g,b,a), lum):
		(h, s, v) = colorsys.rgb_to_hsv(r, g, b)
		(r, g, b) = colorsys.hsv_to_rgb(h, s, v + lum)
		adesklets.context_set_color(r,g,b,a)
		adesklets.image_draw_line(x1, y1, x2, y2, 0)

# <-- EDIT TO MATCH YOUR NEEDS
config = {
	'width': 150,
	'height': 102,
	'bg_color': (0,0,0,180),
	'fg_color': [(255,0,0,255), (255,0,0,255)],
	'grid_x': 4,
	'grid_y': 6,
	'interval': 1
}
# <--

CoreMon(config).pause()
